#include "ConstraintPopulationC.h"
#include "PopulationPoolC.t"
#include "../Common/ArrayC.t"


extern MLCG RandomGenerator;


ConstraintPopulationC::ConstraintPopulationC
(
	UnsignedIntegerT const iSizeOfFitnessHistory,
	SelectionC * iSelection,
	RandomCspC * iProblem
)
{
	UnsignedIntegerT i, j, k;
	ConstraintGenotypeC * ConstraintGenotype;

	SizeOfConstraintPopulation = iProblem->GetNumberOfConstraints ();

	SizeOfFitnessHistory = iSizeOfFitnessHistory;

	Pool = new PopulationPoolC <ConstraintGenotypeC *> (SizeOfConstraintPopulation);

	Problem = iProblem;
	Selection = iSelection;

	GenerationNumber = 0;

	k = 0;
	for (i = 0; i < Problem->GetNumberOfVariables (); i++)
		for (j = i + 1; j < Problem->GetNumberOfVariables (); j++)
		{
			if (Problem->IsConstraint (i, j))
			{
				ConstraintGenotype = new ConstraintGenotypeC (i, j, SizeOfFitnessHistory, Problem);
				Pool->Insert (ConstraintGenotype);
				k++;
			}
		}
} // ConstraintPopulationC::ConstraintPopulationC



ConstraintPopulationC::~ConstraintPopulationC ()
{
	Pool->First ();
	while (!Pool->IsEnd ())
	{
		Pool->SetBookmark ();
		delete Pool->Current ();
		Pool->Next ();
		Pool->DeleteBookmark ();
	}

	delete Pool;

} // ConstraintPopulationC::~ConstraintPopulationC



void const ConstraintPopulationC::InitialiseFitness
(
	EncounterC * iEncounter
)
{
	UnsignedIntegerT i;

	ArrayC <ConstraintGenotypeC *> TempPool (SizeOfConstraintPopulation);

	i = 0;
	Pool->First ();
	while (!Pool->IsEnd ())
	{
		Pool->SetBookmark ();
		TempPool [i] = Pool->Current ();
		TempPool [i]->InitialiseFitness (iEncounter);
		i++;
		Pool->Next ();
		Pool->DeleteBookmark ();
	}

	for (i = 0; i < SizeOfConstraintPopulation; i++)
	{
		Pool->Insert (TempPool [i]);
	}

} // ConstraintPopulationC::Initialise



ConstraintFitnessT const ConstraintPopulationC::GetFitnessOfBestGenotype ()
{
	Pool->First ();
	return (Pool->Current ()->GetFitnessValue ());

} // ConstraintPopulationC::GetFitnessOfBestConstraintGenotype



void const ConstraintPopulationC::Print () const
{
	Pool->First ();
	while (!Pool->IsEnd ())
	{
		Pool->Current ()->Print ();
		Pool->Next ();
	}
	
} // ConstraintPopulationC::Print



void const ConstraintPopulationC::InjectGenotype
(
	ConstraintGenotypeC * iConstraintGenotype
)
{
	Pool->Insert (iConstraintGenotype);

} // ConstraintPopulationC::InjectGenotype



ConstraintGenotypeC * ConstraintPopulationC::FetchGenotype ()
{
	UnsignedIntegerT i;
	ConstraintGenotypeC * ConstraintGenotype;

	Pool->First ();
	for (i = 0; i < Selection->GetIndex (); i++)
	{
		Pool->Next ();
	}
	ConstraintGenotype = Pool->Current ();
	Pool->SetBookmark ();
	Pool->None ();
	Pool->DeleteBookmark ();

	return (ConstraintGenotype);

} // ConstraintPopulationC::SelectGenotype



UnsignedIntegerT const ConstraintPopulationC::GetSize () const
{
	return (SizeOfConstraintPopulation);

} // ConstraintPopulationC::GetSize



UnsignedIntegerT const ConstraintPopulationC::GetFitnessOfBestGenotype () const
{
	Pool->First ();
	return (Pool->Current ()->GetFitnessValue ());

} // ConstraintPopulationC::GetFitnessOfBestGenotype


// eof ConstraintPopulationC.cc
