
#include "EvolutionaryAlgorithmC.h"


StringT EvolutionaryAlgorithmC::FitnessHistorySizeMethods [] =
{
	"none",
	"fixed",
	"populationsize"
};


EvolutionaryAlgorithmC::EvolutionaryAlgorithmC
(
	PositiveIntegerT const iSizeOfSolutionPopulation,
	UnsignedIntegerT const iSizeOfOffspring,
	PositiveIntegerT const iSizeOfGene,
	PositiveIntegerT const iSizeOfFitnessHistory,
	BoolT const iStopConditionGeneration,
	UnsignedIntegerT const iMaximumNumberOfGenerations,
	BoolT const iStopConditionEncountersDone,
	UnsignedIntegerT const iMaximumNumberOfEncountersDone,
	BoolT const iStopConditionGeneratedGenotypes,
	UnsignedIntegerT const iMaximumNumberOfGeneratedGenotypes,
	UnsignedIntegerT const iNumberOfEncounters,
	RealT const iLinearRankingBias,
	ZeroToOneRealRealT const iNormalMutationProbability,
	ZeroToOneRealRealT const iEqualMutationProbability,
	StringT const iFitnessHistorySizeMethod,
	UnsignedIntegerT const iScreenUpdates,
	BoolT const iShowSolutionPopulationOnScreen,
	BoolT const iShowConstraintPopulationOnScreen,
	StatisticsC * iStatistics
)
{
	SizeOfSolutionPopulation = iSizeOfSolutionPopulation;
	SizeOfOffspring = iSizeOfOffspring;
	SizeOfGene = iSizeOfGene;
	SizeOfFitnessHistory = iSizeOfFitnessHistory;
	NumberOfEncounters = iNumberOfEncounters;
	LinearRankingBias = iLinearRankingBias;
	NormalMutationProbability = iNormalMutationProbability;
	EqualMutationProbability = iEqualMutationProbability;
	ScreenUpdates = iScreenUpdates;
	Statistics = iStatistics;
	StopConditionGeneration = iStopConditionGeneration;
	MaximumNumberOfGeneration = iMaximumNumberOfGenerations;
	StopConditionEncountersDone = iStopConditionEncountersDone;
	MaximumNumberOfEncountersDone = iMaximumNumberOfEncountersDone;
	ShowSolutionPopulationOnScreen = iShowSolutionPopulationOnScreen;
	ShowConstraintPopulationOnScreen = iShowConstraintPopulationOnScreen;
	StopConditionGeneratedGenotypes = iStopConditionGeneratedGenotypes;
	MaximumNumberOfGeneratedGenotypes = iMaximumNumberOfGeneratedGenotypes;

	FitnessHistorySizeMethod = none;
	if (strcmp (iFitnessHistorySizeMethod, FitnessHistorySizeMethods [fixed]) == 0)
	{
		FitnessHistorySizeMethod = fixed;
	}
	else
	{
		if (strcmp (iFitnessHistorySizeMethod, FitnessHistorySizeMethods [populationsize]) == 0)
			FitnessHistorySizeMethod = populationsize;
	}

	if (ScreenUpdates != 0)
	{
		cout << "# Generation : Encounters : GeneratedGenotypes : UnsatisfiedConstraints Best Solution : Fitness Best Solution : Fitness Best Constraint \n";
	}

} // EvolutionaryAlgorithmC::EvolutionaryAlgorithmC



EvolutionaryAlgorithmC::~EvolutionaryAlgorithmC ()
{

} // EvolutionaryAlgorithmC::~EvolutionaryAlgorithmC



void const EvolutionaryAlgorithmC::Run
(
	RandomCspC * iProblem
)
{
	UnsignedIntegerT ConstraintFitnessHistorySize;
	UnsignedIntegerT SolutionFitnessHistorySize;

	Problem = iProblem;

	SolutionSelection = new SelectionC (LinearRankingBias, SizeOfSolutionPopulation);
	ConstraintSelection = new SelectionC (LinearRankingBias, Problem->GetNumberOfConstraints ());

	switch (FitnessHistorySizeMethod)
	{
		case fixed:
			SolutionFitnessHistorySize = SizeOfFitnessHistory;
			ConstraintFitnessHistorySize = SizeOfFitnessHistory;
			break;
		case populationsize:
			SolutionFitnessHistorySize = Problem->GetNumberOfConstraints ();
			ConstraintFitnessHistorySize = SizeOfSolutionPopulation;
			break;
		case none:
		default:
			cerr
				<< "Warning: no method for size of FitnesHistory specified, choosing \"fixed\".\n";
			SolutionFitnessHistorySize = SizeOfFitnessHistory;
			ConstraintFitnessHistorySize = SizeOfFitnessHistory;
	}
	
	SolutionPopulation = new SolutionPopulationC
	(
		SizeOfSolutionPopulation, 
		SizeOfOffspring, 
		SizeOfGene, 
		SolutionFitnessHistorySize,
		NormalMutationProbability,
		EqualMutationProbability,
		SolutionSelection,
		Problem
	);

	ConstraintPopulation = new ConstraintPopulationC
	(
		ConstraintFitnessHistorySize,
		ConstraintSelection,
		Problem
	);

	Encounter = new EncounterC (NumberOfEncounters, SolutionPopulation, ConstraintPopulation, Problem);
	/*if (SizeOfSolutionPopulation  > ConstraintPopulation->GetSize ())
		Encounter = new EncounterC (ConstraintPopulation->GetSize () - NumberOfEncounters, SolutionPopulation, ConstraintPopulation, Problem);
	else
		Encounter = new EncounterC (SizeOfSolutionPopulation - NumberOfEncounters, SolutionPopulation, ConstraintPopulation, Problem);*/

	ConstraintPopulation->InitialiseFitness (Encounter);
	SolutionPopulation->InitialiseFitness (Encounter);

	while (!StopCondition ())
	{
		if (ScreenUpdates != 0)
			WriteIntermediateResults ();
		Encounter->DoIt ();
		SolutionPopulation->DoOneGeneration ();
	}

	if (ScreenUpdates != 0)
		WriteResults ();

	if (ScreenUpdates != 0)
	{

		WriteIntermediateResults ();
		cout << "\n";
	}
	
	Statistics->WriteOneProblemTestResult
	(
		SolutionPopulation->GetFoundSolution (),
		SolutionPopulation->GetNumberOfEncountersDone (),
		SolutionPopulation->GetNumberOfGenerations (),
		SolutionPopulation->GetNumberOfGeneratedGenotypes ()
	);

	if (SolutionPopulation->GetFoundSolution ())
	{
		*Statistics << "S\t\t\t";
		SolutionPopulation->GetFoundSolutionGenotype ()->Print (Statistics->GetFileToStoreSolutions ());
		*Statistics << "\n";
	}

	delete SolutionPopulation;
	delete ConstraintPopulation;
	delete Encounter;
	delete ConstraintSelection;
	delete SolutionSelection;

} // EvolutionaryAlgorithmC::Run



void const EvolutionaryAlgorithmC::WriteIntermediateResults () const
{
	if (SolutionPopulation->GetGenerationNumber () % ScreenUpdates == 0)
	{
		WriteResults ();
	}
}



void const EvolutionaryAlgorithmC::WriteResults () const
{
	if (ShowSolutionPopulationOnScreen || ShowConstraintPopulationOnScreen)
		system ("clear");
	if (ShowSolutionPopulationOnScreen)
		SolutionPopulation->Print ();
	if (ShowConstraintPopulationOnScreen)
		ConstraintPopulation->Print ();

	cout
		<< SolutionPopulation->GetGenerationNumber ()
		<< " " << SolutionPopulation->GetNumberOfEncountersDone ()
		<< " " << SolutionPopulation->GetNumberOfGeneratedGenotypes ()
		<< " " << SolutionPopulation->GetNumberOfUnsatisfiedConstraintsOfBestGenotype ()
		<< " " << SolutionPopulation->GetFitnessOfBestGenotype ()
		<< " " << ConstraintPopulation->GetFitnessOfBestGenotype ()
		<< "\n"
	;

	/*if (ShowSolutionPopulationOnScreen || ShowConstraintPopulationOnScreen)
		cout << "\n";
	else
		cout << "\r";*/
} // EvolutionaryAlgorithmC::WriteResults



BoolT const EvolutionaryAlgorithmC::StopCondition () const
{
	return
	(
	 	(StopConditionGeneratedGenotypes && SolutionPopulation->GetNumberOfGeneratedGenotypes () > MaximumNumberOfGeneratedGenotypes) || 
	 	(StopConditionGeneration && SolutionPopulation->GetGenerationNumber() > MaximumNumberOfGeneration) || 
	 	(StopConditionEncountersDone && SolutionPopulation->GetNumberOfEncountersDone() > MaximumNumberOfEncountersDone) || 
		SolutionPopulation->GetFoundSolution ()
	);

} // EvolutionaryAlgorithmC::StopCondition



BoolT const EvolutionaryAlgorithmC::GetFoundSolution ()
{
	return (FoundSolution);
	
}  // EvolutionaryAlgorithmC::GetFoundSolution



UnsignedIntegerT const EvolutionaryAlgorithmC::GetNumberOfEncountersDone ()
{
	return (NumberOfEncountersDone);

} // EvolutionaryAlgorithmC::GetNumberOfConstraints



// eof EvolutionaryAlgorithmC.cc
