#include "SolutionGenotypeC.h"
#include "EncounterC.h"
#include "FitnessC.cc"

extern MLCG RandomGenerator;


static UnsignedIntegerT IdMaker = 0;

SolutionGenotypeC::SolutionGenotypeC
(
	UnsignedIntegerT iSizeOfGene,
	UnsignedIntegerT iSizeOfFitnessHistory,
	RandomCspC * iProblem
)
{
	SolutionGeneNumberT i;

	SizeOfGene = iSizeOfGene;
	Data = new ArrayC <SolutionGeneT> (SizeOfGene);

	Fitness = new FitnessC <SolutionGeneT> (iSizeOfFitnessHistory);

	Id = IdMaker++;

	Problem = iProblem;

	NumberOfUnsatisfiedConstraints = 0;
	Dirty = true;

	for (i = 0; i < SizeOfGene; i++)
	{
		(*Data) [i] = RandomGenerator.asLong () % Problem->GetDomainSize (i);
	}

} // SolutionGenotypeC::SolutionGenotypeC



/*SolutionGenotypeC::SolutionGenotypeC
(
	SolutionGenotypeC & const iSolutionGenotype
)
{
} // SolutionGenotypeC::SolutionGenotypeC
*/


SolutionGenotypeC::~SolutionGenotypeC ()
{
	//cerr << "Deleting Genotype: ";
	//cerr << Id << endl;
	delete Data;

	delete Fitness;

} // SolutionGenotypeC::~SolutionGenotypeC



void const SolutionGenotypeC::InitialiseFitness
(
	EncounterC * iEncounter
)
{
	iEncounter->DoIt (this);

} // SolutionGenotypeC::InitialiseFitness



SolutionGenotypeC const & SolutionGenotypeC::operator=
(
	SolutionGenotypeC const & iSolutionGenotype
)
{
	if (this != &iSolutionGenotype)
	{
		*Data = iSolutionGenotype.Data [0];
		*Fitness = iSolutionGenotype.Fitness [0];
		SizeOfGene = iSolutionGenotype.SizeOfGene;
	}

	return (*this);


} // SolutionGenotypeC::operator=



BoolT const SolutionGenotypeC::operator==
(
	SolutionGenotypeC const & iSolutionGenotype
) const
{
	if (*Data == iSolutionGenotype.Data [0])
		return (true);
	else
		return (false);

} // SolutionGenotypeC::operator==



void const SolutionGenotypeC::Print() const
{
	SolutionGeneNumberT i;

	for (i = 0; i < GetSizeOfGene (); i++)
		cout << setw(3) << (*Data) [i];

} // SolutionGenotypeC::Print



void const SolutionGenotypeC::Print
(
	FileC & iFile
) const
{
	SolutionGeneNumberT i;

	for (i = 0; i < GetSizeOfGene (); i++)
		iFile << (*Data) [i] << " ";

} // SolutionGenotypeC::Print



UnsignedIntegerT const SolutionGenotypeC::GetNumberOfUnsatisfiedConstraints ()
{
	UnsignedIntegerT i, j;

	if (Dirty)
	{
		NumberOfUnsatisfiedConstraints = 0;
		i = 0;
		while (i < GetSizeOfGene ())
		{
			j = i + 1;
			while (j < GetSizeOfGene ())
			{
				if (Problem->IsConflict (i, j, GetGene (i), GetGene (j)))
					NumberOfUnsatisfiedConstraints++;
				j++;
			}
			i++;
		}
		Dirty = false;
	}

	return (NumberOfUnsatisfiedConstraints);

} // SolutionGenotypeC::GetNumberOfUnsatisfiedConstraints



SolutionFitnessT const SolutionGenotypeC::GetFitnessValue () const
{
	return (Fitness->GetFitness ());
	
} // SolutionGenotypeC::GetFitnessValue



void const SolutionGenotypeC::SetGene
(
	SolutionGeneNumberT iGeneNumber,
	SolutionGeneT iGene
)
{
	Dirty = true;

	(*Data) [iGeneNumber] = iGene; 
	
} // SolutionGenotypeC::SetGene



SolutionGeneT const SolutionGenotypeC::GetGene
(
	SolutionGeneNumberT iGeneNumber
) const
{
	return ((*Data) [iGeneNumber]);

} // SolutionGenotypeC::GetGene



/*
 * Two-point surrogate parents crosover.
 *
 */

void const SolutionGenotypeC::Crossover
(
	SolutionGenotypeC const & iOtherParent,
	SolutionGenotypeC & oChild1,
	SolutionGenotypeC & oChild2
)
{
	SolutionGeneT c1;
	SolutionGeneT c2;
	SolutionGeneT i;
	SolutionGeneT j;

	i = 0;
	while ( (i < GetSizeOfGene ()) && (GetGene (i) == iOtherParent.GetGene (i)) )
		i++;
	
	
	if (i == GetSizeOfGene ())
	{
		oChild1 = *this;
		oChild2 = iOtherParent;
	}
	else
	{

		j = GetSizeOfGene () - 1;
		while ( (j > 0) && (GetGene (j) == iOtherParent.GetGene (j)) )
			j--;

		c1 = RandomGenerator.asLong () % (j - i + 1) + i;
		c2 = RandomGenerator.asLong () % (j - i + 1) + i;
		if (c1 > c2)
		{
			i = c1;
			c1 = c2;
			c2 = i;
		}
		
		oChild1 = *this;
		oChild2 = iOtherParent;

		for (i = c1; i < c2; i++)
		{
			oChild2.SetGene (i, GetGene(i));
			oChild1.SetGene (i, iOtherParent.GetGene (i));
		}

		// DEBUG // Print (); cout << "\n"; oChild1.Print (); cout << "\n"; iOtherParent.Print (); cout << "\n"; oChild2.Print (); cout << "\n"; cout << "\n";
	}

} // SolutionGenotypeC::Crossover


void const SolutionGenotypeC::Mutate
(
	RealT MutationProbability
)
{
	VariableT allel;

/*	ONE-ALLEL MUTATION
	allel = RandomGenerator.asLong() % SizeOfGene;

	SetGene (allel, RandomGenerator.asLong() % Problem->GetDomainSize (allel));*/

	for (allel = 0; allel < GetSizeOfGene (); allel++)
		if ((RealT) (RandomGenerator.asLong () % 10000) < MutationProbability * 10000)
		{
			SetGene (allel, RandomGenerator.asLong() % Problem->GetDomainSize (allel));
		}

} // SolutionGenotypeC::Mutate



SolutionGeneNumberT const SolutionGenotypeC::GetSizeOfGene () const
{
	return (SizeOfGene);

} // SolutionGenotypeC::GetSizeOfGene



void const SolutionGenotypeC::UpdateFitness
(
	BoolT iReward
)
{
	if (iReward)
		Fitness->UpdateFitness (1);
	else
		Fitness->UpdateFitness (0);

} // SolutionGenotypeC::UpdateHistory

// eof SolutionGenotypeC.cc
