
/*
 * ConfigurationC.cc - A general configuration file reader with classes.
 *
 */

#include "ConfigurationC.h"


ConfigurationC::ConfigurationC
(
	StringT const iFileName
)
{
	char EqualSine;
	ifstream FileData;
	StringT s;
	StringT Class = "";

	NumberOfLabels = 0;
	LargestString = 0;

	FileData.open (iFileName);
	if (!FileData.is_open ())
	{
		cerr << "Error: The inifile \"" << iFileName << "\" could not be opened." << endl;
		exit (1);
	}

	FileName = strdup (iFileName);

	s = new char [200];
	while (!FileData.eof ())
	{
		FileData.getline (s, 200);
		if (LargestString < strlen (s))
			LargestString = strlen (s);
		if (strchr (s, '=') != 0)
			NumberOfLabels++;
	}
	delete s;
	Items = new ItemsT [NumberOfLabels];

	FileData.close ();
	FileData.open (iFileName);

	s = new char [LargestString];
	UnsignedIntegerT i = 0;
	while (!FileData.eof () && (i <= NumberOfLabels))
	{
		//FileData.getline (s, LargestString);
		FileData >> s;

		if ((s [0] == '[' ) && (s [strlen(s) - 1] == ']'))
		{
			Class = strdup (s);
		}
		else
			if (s [0] != '\0')
			{
				Items [i].Label = new char [strlen (Class) + strlen (s) + 2];
				strcpy (Items [i].Label, Class);
				Items [i].Label = strcat (Items [i].Label, ".");
				Items [i].Label = strcat (Items [i].Label, s);
				FileData >> EqualSine;
				if (EqualSine != '=')
				{
					cerr << "Error: reading in \"" << iFileName << "\" while looking at \"" << s << "\"." << endl;
					exit (1);
				}

				FileData >> s;
				Items [i].Data = strdup (s);
				i++;
			}
	}
	delete s;

	if (!FileData.eof ())
	{
		cerr << "Error: More labels read in then should be in file \"" << iFileName << "\", please check the file for errors." << endl;
		FileData.close ();
		exit (1);
	}

	FileData.close ();
} // ConfigurationC::ConfigurationC



ConfigurationC::~ConfigurationC ()
{
	UnsignedIntegerT i;

	for (i = 0 ; i < NumberOfLabels; i++)
	{
		delete Items [i].Label;
		delete Items [i].Data;
	}
	delete FileName;
	delete Items;

} // ConfigurationC::~ConfigurationC



UnsignedIntegerT const ConfigurationC::GetUnsignedIntegerItem
( 
	StringT const iClass, 
	StringT const iItem
) const
{
	IntegerT i;

	i = atoi (GetStringItem (iClass, iItem));

	if (i < 0)
	{
			cerr << "Warning: label \"" << iItem 
					<< "\" of class \"" << iClass
					<< "\" in file \"" << FileName 
					<< "\" is not allowed to have a negative value, defaulting to 0." << endl;
			return (0);
	}
	else
		return ( (UnsignedIntegerT) i);

} // ConfigurationC::GetIntegerItem



IntegerT const ConfigurationC::GetIntegerItem
( 
	StringT const iClass, 
	StringT const iItem
) const
{
	return (atoi (GetStringItem (iClass, iItem)));

} // ConfigurationC::GetIntegerItem



PositiveIntegerT const ConfigurationC::GetPositiveIntegerItem
(
	StringT const iClass,
	StringT const iItem
) const
{
	PositiveIntegerT i;

	i = GetUnsignedIntegerItem (iClass, iItem);

	if (i == 0)
	{
		cerr << "Warning: label \"" << iItem 
				<< "\" of class \"" << iClass
				<< "\" in file \"" << FileName 
				<< "\" is not allowed to have the value zero, defaulting to 1." << endl;
		return (1);
	}

	return ( (PositiveIntegerT) i);

} // ConfigurationC::GetRealItem



RealT const ConfigurationC::GetRealItem
(
	StringT const iClass,
	StringT const iItem
) const
{
	return (atof (GetStringItem (iClass, iItem)));

} // ConfigurationC::GetRealItem



RealT const ConfigurationC::GetZeroToOneRealItem
(
	StringT const iClass,
	StringT const iItem
) const
{
	RealT r;

	r = atof (GetStringItem (iClass, iItem));

	if ((r < 0) || (r > 1))
	{
		cerr << "Warning: label \"" << iItem 
				<< "\" of class \"" << iClass
				<< "\" in file \"" << FileName 
				<< "\" must have a value from the range 0 to 1, defaulting to 0." << endl;

		return (0);
	}

	return (r);

} // ConfigurationC::GetRealItem



StringT const ConfigurationC::GetStringItem
(
	StringT const iClass, 
	StringT const iItem
) const
{
	UnsignedIntegerT i;
	StringT Search = new char [2 * LargestString + 4];

	strcpy (Search, "[");
	Search = strcat (Search, iClass);
	Search = strcat (Search, "].");
	Search = strcat (Search, iItem);

	i = 0;
	while ((i < NumberOfLabels) && (strcmp (Search, Items [i].Label) != 0))
		i++;

	delete Search;

	if (i < NumberOfLabels)
		return (Items [i]).Data;
	else
	{
		cerr << "Warning: label \"" << iItem << "\" not found in \"" << FileName << "\"." << endl;
		return ("");
	}
} // ConfigurationC::GetStringItem



BoolT const ConfigurationC::GetBoolItem
(
	StringT const iClass,
	StringT const iItem
) const
{
	if (strcmp ("true", GetStringItem (iClass, iItem)) == 0)
		return (true);

	if (strcmp ("false", GetStringItem (iClass, iItem)) == 0)
		return (false);

	switch (GetIntegerItem (iClass, iItem))
	{
		case 0:
			return (false);
		case 1:
			return (true);
		default:
			cerr << "Warning: label \"" << iItem 
					<< "\" of class \"" << iClass
					<< "\" in file \"" << FileName 
					<< "\" doesn't have a boolean (0 or 1) value, choosing false." << endl;
			return (false);
	}
} // ConfigurationC::GetBoolItem



void const ConfigurationC::Print () const
{
	for (UnsignedIntegerT i = 0; i < NumberOfLabels; i++)
		cout << Items [i].Label << " = " << Items [i].Data << endl;

} // ConfigurationC::Print

// eof ConfigurationC.cc
