
#include "ExperimentC.h"

MLCG RandomGenerator;

StringT ExperimentC::ProblemGeneratorMethods [] =
{
	"none",
	"Dozier",
	"Prosser"
};

ExperimentC::ExperimentC (StringT iConfigurationFileNam)
{
	ConfigurationFile = new ConfigurationC (iConfigurationFileNam);

	// ConfigurationFile->Print ();

	Statistics = new StatisticsC
	(
		ConfigurationFile->GetStringItem ("Output", "IntermediateFileName"),
		ConfigurationFile->GetStringItem ("Output", "RunStatsFileName"),
		ConfigurationFile->GetStringItem ("Output", "SuccessRateFileName"),
		ConfigurationFile->GetStringItem ("Output", "EncountersDoneFileName"),
		ConfigurationFile->GetStringItem ("Output", "GeneratedGenotypesFileName"),
		ConfigurationFile->GetStringItem ("Output", "GenerationsFileName"),
		ConfigurationFile->GetStringItem ("Output", "Directory"),
		ConfigurationFile->GetBoolItem ("Output", "OverwriteOutputFiles")
	);

	RandomGenerator.seed1 (ConfigurationFile->GetUnsignedIntegerItem ("Experiment", "RandomSeed1"));
	RandomGenerator.seed2 (ConfigurationFile->GetUnsignedIntegerItem ("Experiment", "RandomSeed2"));

	if (strcmp (ProblemGeneratorMethods [Dozier], ConfigurationFile->GetStringItem ("Problem", "Method")) == 0)
	{
		ProblemGeneratorMethod = Dozier;
	}
	else
	{
		if (strcmp (ProblemGeneratorMethods [Prosser], ConfigurationFile->GetStringItem ("Problem", "Method")) == 0)
			ProblemGeneratorMethod = Prosser;
		else
			ProblemGeneratorMethod = Dozier;
	}

	DoProblemParametersettingsExperiment ();
}



void const ExperimentC::DoProblemParametersettingsExperiment ()
{
	StringT hostname;

	hostname = new char [40];
	if (gethostname (hostname, 38) != 0)
		strcpy (hostname, "unknown");

	*Statistics 
		<< "C Starting Coga experiment\n"
   		<< "C\t" << ConfigurationFile->GetStringItem ("Experiment", "Description") << "\n"
		<< "C\tRunning on " << hostname << "\n"
		<< "C\tExperiment parameters\n"
		<< "P\t\tRandomSeed1"
		<< ConfigurationFile->GetUnsignedIntegerItem ("Experiment", "RandomSeed1")
		<< "P\t\tRandomSeed2"
		<< ConfigurationFile->GetUnsignedIntegerItem ("Experiment", "RandomSeed2")
		<< "C\tEvolutionary Algorithm parameters\n"
		<< "P\t\tPopulationSize"
		<< ConfigurationFile->GetPositiveIntegerItem ("EvolutionaryAlgorithm", "PopulationSize")
		<< "P\t\tOffspringSize"
		<< ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "OffspringSize")
		<< "P\t\tGeneSize"
		<< ConfigurationFile->GetPositiveIntegerItem ("EvolutionaryAlgorithm", "GeneSize")
		<< "P\t\tFitnessHistorySizeMethod = "
		<< ConfigurationFile->GetStringItem ("EvolutionaryAlgorithm", "FitnessHistorySizeMethod") << "\n"
		<< "P\t\tFitnessHistorySize"
		<< ConfigurationFile->GetPositiveIntegerItem ("EvolutionaryAlgorithm", "FitnessHistorySize")
		<< "P\t\tUseMaximumGenerations"
		<< ConfigurationFile->GetBoolItem ("EvolutionaryAlgorithm", "UseMaximumGenerations")
		<< "P\t\tMaximumGenerations"
		<< ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "MaximumGenerations")
		<< "P\t\tUseMaximumEncountersDone"
		<< ConfigurationFile->GetBoolItem ("EvolutionaryAlgorithm", "UseMaximumEncountersDone")
		<< "P\t\tMaximumEncountersDone"
		<< ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "MaximumEncountersDone")
		<< "P\t\tUseMaximumGeneratedGenotypes"
		<< ConfigurationFile->GetBoolItem ("EvolutionaryAlgorithm", "UseMaximumGeneratedGenotypes")
		<< "P\t\tMaximumGeneratedGenotypes"
		<< ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "MaximumGeneratedGenotypes")
		<< "P\t\tNumberOfEncounters"
		<< ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "NumberOfEncounters")
		<< "P\t\tLinearRankingBias"
		<< ConfigurationFile->GetRealItem ("EvolutionaryAlgorithm", "LinearRankingBias")
		<< "P\t\tNormalMutationProbability"
		<< ConfigurationFile->GetZeroToOneRealItem ("EvolutionaryAlgorithm", "NormalMutationProbability")
		<< "P\t\tEqualMutationProbability"
		<< ConfigurationFile->GetZeroToOneRealItem ("EvolutionaryAlgorithm", "EqualMutationProbability")
	;

	for (RealT Connectivity = ConfigurationFile->GetZeroToOneRealItem ("Problem", "ConnectivityBegin");
		Connectivity <= ConfigurationFile->GetZeroToOneRealItem ("Problem", "ConnectivityEnd");
		Connectivity += ConfigurationFile->GetZeroToOneRealItem ("Problem", "ConnectivityStepsize"))
	{
		for (RealT Tightness = ConfigurationFile->GetZeroToOneRealItem ("Problem", "TightnessBegin");
			Tightness <= ConfigurationFile->GetZeroToOneRealItem ("Problem", "TightnessEnd");
			Tightness += ConfigurationFile->GetZeroToOneRealItem ("Problem", "TightnessStepsize"))
		{
			DoDifferentProblemsTest
			(
				ConfigurationFile->GetIntegerItem ("Problem", "NumberOfVariables"),
				ConfigurationFile->GetIntegerItem ("Problem", "DomainSize"),
				ConfigurationFile->GetBoolItem ("Problem", "RandomDomainSize"),
				Connectivity,
				Tightness
			);

			Statistics->WriteOneParameterResult (Connectivity, Tightness);

		}
	}

	Statistics->WriteExperimentResult ();
	*Statistics 
		<< "C Ending Coga experiment\n\n\n\n";

	delete hostname;

} // ExperimentC::DoProblemParametersettings



void const ExperimentC::DoDifferentProblemsTest
(
	PositiveIntegerT iNumberOfVariables,
	PositiveIntegerT iDomainSize,
	BoolT iRandomDomainSize,
	ZeroToOneRealRealT iConnectivity,
	ZeroToOneRealRealT iTightness
)
{
	*Statistics
			<< "C\tProblem Parameters\n"
			<< "P\t\tMethod = "
			<< ConfigurationFile->GetStringItem ("Problem", "Method") << "\n"
			<< "P\t\tNumberOfVariables"
			<< iNumberOfVariables
			<< "P\t\tDomainSize"
			<< iDomainSize
			<< "P\t\tRandomDomainSize"
			<< iRandomDomainSize
			<< "P\t\tConnectivity"
			<< iConnectivity
			<< "P\t\tTightness"
			<< iTightness
			<< "P\t\tRandomSeed1"
			<< ConfigurationFile->GetUnsignedIntegerItem ("Problem", "RandomSeed1")
			<< "P\t\tRandomSeed2"
			<< ConfigurationFile->GetUnsignedIntegerItem ("Problem", "RandomSeed2")
		;
		
	Problem->InitRandomGenerator
	(
		ConfigurationFile->GetUnsignedIntegerItem ("Problem", "RandomSeed1"),
		ConfigurationFile->GetUnsignedIntegerItem ("Problem", "RandomSeed2")
	);

	for (UnsignedIntegerT i = 0; i < ConfigurationFile->GetPositiveIntegerItem ("Experiment", "ProblemsPerParameterSetting"); i++)
	{
		DoMultipleTimesOneProblemTest
		(
			ConfigurationFile->GetIntegerItem ("Experiment", "ProblemInstances"),
			iNumberOfVariables,
			iDomainSize,
			iRandomDomainSize,
			iConnectivity,
			iTightness
		);
	}

} // ExperimentC::DoDifferentProblemsTest


void const ExperimentC::DoMultipleTimesOneProblemTest
(
 	PositiveIntegerT iNumberOfTimes,
	PositiveIntegerT iNumberOfVariables,
	PositiveIntegerT iDomainSize,
	BoolT iRandomDomainSize,
	ZeroToOneRealRealT iConnectivity,
	ZeroToOneRealRealT iTightness
)
{
	StringT temp;

	switch (ProblemGeneratorMethod)
	{
		case Prosser:
			Problem = new ProsserRandomCspC
			(
				iNumberOfVariables,
				iDomainSize,
				iRandomDomainSize,
				iTightness,
				iConnectivity
			);
			break;
		case Dozier:
		default:
			Problem = new DozierRandomCspC
			(
				iNumberOfVariables,
				iDomainSize,
				iRandomDomainSize,
				iTightness,
				iConnectivity
			);
	}

	Problem->ProduceProblem ();

	temp = new char [strlen (ConfigurationFile->GetStringItem ("Output", "Directory")) + 1 + strlen (ConfigurationFile->GetStringItem ("Problem", "LastProblemFileName"))];
	temp = strcpy (temp, ConfigurationFile->GetStringItem ("Output", "Directory"));
	temp = strcat (temp, ConfigurationFile->GetStringItem ("Problem", "LastProblemFileName"));
	Problem->SaveToFile(temp);
	delete temp;

	*Statistics
			<< "C\t\tGenerating a new problem\n"
			<< "P\t\tNumberOfConstraints"
			<< Problem->GetNumberOfConstraints ()
			;

	for (UnsignedIntegerT i = 0; i < iNumberOfTimes; i++)
	{
		EvolutionaryAlgorithm = new EvolutionaryAlgorithmC
		(
			ConfigurationFile->GetPositiveIntegerItem ("EvolutionaryAlgorithm", "PopulationSize"),
			ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "OffspringSize"),
			ConfigurationFile->GetPositiveIntegerItem ("EvolutionaryAlgorithm", "GeneSize"),
			ConfigurationFile->GetPositiveIntegerItem ("EvolutionaryAlgorithm", "FitnessHistorySize"),
			ConfigurationFile->GetBoolItem ("EvolutionaryAlgorithm", "UseMaximumGenerations"),
			ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "MaximumGenerations"),
			ConfigurationFile->GetBoolItem ("EvolutionaryAlgorithm", "UseMaximumEncountersDone"),
			ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "MaximumEncountersDone"),
			ConfigurationFile->GetBoolItem ("EvolutionaryAlgorithm", "UseMaximumGeneratedGenotypes"),
			ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "MaximumGeneratedGenotypes"),
			ConfigurationFile->GetUnsignedIntegerItem ("EvolutionaryAlgorithm", "NumberOfEncounters"),
			ConfigurationFile->GetRealItem ("EvolutionaryAlgorithm", "LinearRankingBias"),
			ConfigurationFile->GetZeroToOneRealItem ("EvolutionaryAlgorithm", "NormalMutationProbability"),
			ConfigurationFile->GetZeroToOneRealItem ("EvolutionaryAlgorithm", "EqualMutationProbability"),
			ConfigurationFile->GetStringItem ("EvolutionaryAlgorithm", "FitnessHistorySizeMethod"),
			ConfigurationFile->GetUnsignedIntegerItem ("Output", "ScreenUpdates"),
			ConfigurationFile->GetBoolItem ("Output", "ShowSolutionPopulationOnScreen"),
			ConfigurationFile->GetBoolItem ("Output", "ShowConstraintPopulationOnScreen"),
			Statistics
		);

		EvolutionaryAlgorithm->Run (Problem);

		delete EvolutionaryAlgorithm;
	}

	delete Problem;
	
} // ExperimentC::DoMultipleTimesOneProblemTest



ExperimentC::~ExperimentC ()
{
	delete ConfigurationFile;
	delete Statistics;

} // ExperimentC::~ExperimentC




// eof ExperimentC.cc
